;'50Hurts' Scrolling Engine.
;---------------------------

;by Jason 'Joefish' Railton.
;---------------------------

;based on techniques developed by Jonathan 'Joffa' Smith.
;--------------------------------------------------------

;128K only.
;----------


	org 33025			;Where the code is going to go in memory
entry_point          	;Label for where to start execution
	di

	ld sp,49152			;Miss the RET that will be at 49087

	;Show screen 0 but page screen 1 in at 49152:
	ld bc,32765			;Output port for memory paging
	ld a,7
	out (c),a			;View screen 0, with screen 1 in high memory

;Configure interrupt vector table,
;set 32768..33024 (257 bytes) to 255
	ld hl,32768		;128*256=32768
  	ld de,32769
	ld (hl),191		;write once
	ld bc,256       ;make 256 copies
	ldir

;Interrupts will go to 191*256+191 = 49087
;Just stick a RET at this address.  The interrupt only exists
;to be able to time things.
;Note that a simple RET (with no EI) will leave interrupts
;diabled on return.  They must be re-enabled before any use
;of HALT for timing.
	ld a,$C9		;RET
	ld (49087),a

;Install interrupts:
	ld a,128		;high byte of address table
	ld i,a
	im 2

	ld a,0
	out (254),a

	;Clear screen 0 pixels:
	ld hl,16384			;Start of screen 0
	ld de,16385			;One byte later
	ld A,0
	ld(hl),A			;Stick in a 0
	ld bc,6144-1    	;Count bytes for screen 1
	ldir				;Copy from (hl) to (de) and repeat bc times.

	;Clear screen 1 pixels:
	ld hl,49152			;Start of screen 1, when paged in
	ld de,49153			;One byte later
	ld A,0
	ld(hl),A			;Stick in a 0
	ld bc,6144-1    	;Count bytes for screen less 1
	ldir				;Copy from (hl) to (de) and repeat bc times.


	ld de,37567
	ld hl,block_graphics
	ld bc,720
	ldir


	ld hl,37567
	ld de,38287
	ld a,15
block_scroll_loop
	ld c,5
block_set_loop
	ld b,16
block_lines_loop
	push af
	push bc

	ld bc,8
	ldir

	ex de,hl
	dec hl
	push hl
	sla (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	dec hl
	rl (hl)
	ld a,0
	rl a
	pop hl
	or (hl)
	ld (hl),a
	inc hl
	ex de,hl

	pop bc
	pop af
	dec b
	jp nz,block_lines_loop

	push af
	push bc
	cp 8
	jp nz,no_shift_attrs
shift_attrs
	ld a,(hl)
	inc hl
	ld bc,7
	ldir
	ld (de),a
	inc de
	ld a,(hl)
	inc hl
	ld bc,7
	ldir
	ld (de),a
	inc de
	jp shift_attrs_done
no_shift_attrs
	ld bc,16
	ldir
shift_attrs_done
	pop bc
	pop af

	dec c
	jp nz,block_set_loop

	dec a
	jp nz,block_scroll_loop


	ld hl,pattern_graphic_a
	ld ix,37567
	ld iy,43327

	ld d,16
pattern_row_loop
	push ix
	push iy
	ld a,(hl)
	ld b,254
	ld c,0

	ld e,4
pattern_scroll_loop
	push de

	ld d,2
pattern_dup_loop

	ld e,4
pattern_block_loop

	push ix
	push iy

	ld (ix),a
	inc ix
	ld (ix),a
	inc ix
	ld (ix),a
	inc ix
	push af
	and b
	or (ix)
	ld (ix),a
	pop af
	inc ix
	inc ix
	inc ix
	inc ix
	push af
	and c
	or (ix)
	ld (ix),a
	pop af

	push af
	push bc

	rrc a
	rrc a
	rrc a
	rrc a

	ld (iy),a
	inc iy
	ld (iy),a
	inc iy
	push af
	and b
	or (iy)
	ld (iy),a
	pop af
	inc iy
	inc iy
	inc iy
	inc iy
	push af
	and c
	or (iy)
	ld (iy),a
	pop af
	inc iy
	ld (iy),a

	pop bc
	pop af

	pop iy
	pop ix

	push bc
	ld bc,144
	add ix,bc
	add iy,bc
	pop bc
	dec e
	jp nz,pattern_block_loop

	ccf
	sla b
	rl c

	push bc
	ld bc,144
	add ix,bc
	add iy,bc
	pop bc
	dec d
	jp nz,pattern_dup_loop

	pop de
	rlc a

	dec e
	jp nz,pattern_scroll_loop

	pop iy
	pop ix
	ld bc,8
	add ix,bc
	add iy,bc
	inc hl
	dec d
	jp nz,pattern_row_loop


	ld hl,map_data
	ld bc,1751
map_transform_loop
	push bc

	ld a,(hl)
	inc hl
	cp 0
	ld c,0
	jp z,a0_eq_0
	ld c,2
a0_eq_0
	ld a,(hl)
	dec hl
	cp 0
	ld a,0
	jp z,a1_eq_0
	ld a,1
a1_eq_0
	add a,c
	ld c,a
	ld b,0
	ex de,hl
	ld hl,push_table
	add hl,bc
	ld a,(hl)
	ex de,hl
	ld (hl),a
	inc hl

	pop bc
	dec bc
	ld a,b
	or c
	jp nz,map_transform_loop



	ld a,0
	ld (px_offset),a
	ld a,0
	ld (px_offset),a

flicker_loop
	ld sp,49152
	ei					;Because calling the last interrupt will have diabled interrupts
	halt				;Wait for 50Hz interrupt, which RETurns immediately
	ei					;Because calling the last interrupt will have diabled interrupts
	halt				;Wait for 50Hz interrupt, which RETurns immediately

	;ld a,0
	;out (254),a

	ld bc,32765			;Output port for memory paging
screen_swap_point
	ld a,7
	out (c),a					;View one screen, with the other in high memory
	xor 10						;Swap both
	ld (screen_swap_point+1),a	;Set up for next time

	ld a,(map_offset)
	ld c,a
	ld b,0
	ld hl,map_data
	add hl,bc
	ld bc,16
	add hl,bc
	ld ix,push_set_list
	ld a,12
map_load_loop
	ld e,(ix)
	inc ix
	ld d,(ix)
	inc ix

	ex af,af'
	ld b,16
push_load_loop
	ld a,(hl)
	ld (de),a
	dec hl
	inc de
	djnz push_load_loop
	ex af,af'

	ld bc,162
	add hl,bc
	dec a
	jp nz,map_load_loop

	ld a,(map_offset)
	and 64
	jp nz,use_blocks_1
use_blocks_0
	ld a,0
	ld (alt_block_01),a
	ld (alt_block_02),a
	ld (alt_block_05),a
	ld (alt_block_06),a
	ld (alt_block_09),a
	ld (alt_block_10),a
	ld hl,37567
	ld ix,37855
	jp use_blocks_done
use_blocks_1
	ld a,$D9 	;EXX
	ld (alt_block_01),a
	ld (alt_block_02),a
	ld (alt_block_05),a
	ld (alt_block_06),a
	ld (alt_block_09),a
	ld (alt_block_10),a
	ld hl,37711
	ld ix,37999
use_blocks_done
	ld iy,38143
	ld a,(px_offset)
	ld c,0
	ld b,a
	sla b
	add hl,bc
	add ix,bc
	add iy,bc
	ld b,a
	srl b
	rr c
	add hl,bc
	add ix,bc
	add iy,bc
	srl b
	rr c
	add hl,bc
	add ix,bc
	add iy,bc
	srl b
	rr c
	srl b
	rr c
	add hl,bc
	add ix,bc
	add iy,bc
	ld (src_sp_set_a+1),hl
	ld (src_sp_set_b+1),ix
	ld (src_sp_set_c+1),iy

	ld ix,screen_task_list
	ld a,18
big_push_loop
	ex af,af'

src_sp_set_a
	ld sp,38847
	pop af
	pop bc
	pop de
	pop hl
	ld (src_sp_set_a+1),sp
	exx			;Not using alternate AF
src_sp_set_b
	ld sp,39103
	pop af		;Same pattern as before
	pop bc
	pop de
	pop hl
	ld (src_sp_set_b+1),sp

alt_block_00
	nop;exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_00
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_01
	nop;exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_01
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_02
	nop;exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_02
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_03
	exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_03
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_04
	exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_04
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_05
	nop;exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_05
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_06
	nop;exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_06
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_07
	exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_07
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_08
	exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_08
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_09
	nop;exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_09
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

alt_block_10
	nop;exx

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_10
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

	ex af,af'
	dec a
	jp nz,big_push_loop

	ld a,18
small_push_loop
	ex af,af'

src_sp_set_c
	ld sp,39359
	pop af
	pop bc
	pop de
	pop hl
	ld (src_sp_set_c+1),sp

	ld sp,ix
	pop iy
	ld sp,iy
	inc ix
	inc ix

push_set_11
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af
	push hl
	push de
	push bc
	push af
		push hl
		push de
		push bc
		push af

	ex af,af'
	dec a
	jp nz,small_push_loop


	ld a,(px_offset)
	inc a
	inc a
	and 15
	ld (px_offset),a
	cp 0
	jp nz,scroll_done
	ld a,(map_offset)
	inc a
	and 127
	ld (map_offset),a
scroll_done

	;ld a,1
	;out (254),a

	jp flicker_loop		;Repeat

	;ret				;No return to BASIC

big_push_loop_count
	defb 16

px_offset
	defb 0
map_offset
	defb 0

block_graphics
block_graphic_a
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000001b,11101111b,11101111b,11101111b,11101111b
	defb 00000000b,00000000b,00000000b,00000001b,11101111b,11101111b,11101111b,11101111b
	defb 00000000b,00000000b,00000000b,00000001b,11101111b,11101111b,11101111b,11101111b
	defb 00000000b,00000000b,00000000b,00000001b,11101111b,11101111b,11101111b,11101111b
	defb 00000000b,00000000b,00000000b,00000001b,11101111b,11101111b,11101111b,11101111b
	defb 00000000b,00000000b,00000000b,00000001b,11101111b,11101111b,11101111b,11101111b
	defb 00000000b,00000000b,00000000b,00000000b,11101111b,11101111b,11101111b,11101110b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,11011111b,01111011b,11011111b,01111010b
	defb 00000000b,00000000b,00000000b,00000000b,11011111b,01111011b,11011111b,01111010b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,01110111b,01110111b,01110111b,01110110b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,11011101b,10110110b,11011101b,10110110b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b

	defb 00000101b,00000101b,00000101b,01000111b,01000111b,01000111b,01000111b,01000111b
	defb 00000101b,00000101b,00000101b,00000111b,00000111b,00000111b,00000111b,00000111b

block_graphic_b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000001b,11111111b,11111111b,11111111b,11111111b
	defb 00000000b,00000000b,00000000b,00000001b,11111110b,11111110b,11111110b,11111111b
	defb 00000000b,00000000b,00000000b,00000001b,11111111b,11111111b,11111111b,11111111b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000001b,11010000b,00000111b,11010000b,00000111b
	defb 00000000b,00000000b,00000000b,00000000b,11101000b,00001110b,11101000b,00001110b
	defb 00000000b,00000000b,00000000b,00000000b,01110100b,00011100b,01110100b,00011100b
	defb 00000000b,00000000b,00000000b,00000000b,00111010b,00111010b,00111010b,00111010b
	defb 00000000b,00000000b,00000000b,00000000b,00011101b,01110100b,00011101b,01110100b
	defb 00000000b,00000000b,00000000b,00000000b,00001110b,11101000b,00001110b,11101000b
	defb 00000000b,00000000b,00000000b,00000000b,00000111b,11010000b,00000111b,11010000b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000001b,11111111b,11111111b,11111111b,11111111b
	defb 00000000b,00000000b,00000000b,00000001b,11111110b,11111110b,11111110b,11111111b
	defb 00000000b,00000000b,00000000b,00000001b,11111111b,11111111b,11111111b,11111111b

	defb 00000101b,00000101b,00000101b,01000111b,01000111b,01000111b,01000111b,01000111b
	defb 00000101b,00000101b,00000101b,00000110b,00000110b,00000110b,00000110b,00000110b

block_graphic_c
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,10111111b,10111111b,10111111b,10111110b
	defb 00000000b,00000000b,00000000b,00000000b,10111111b,10111111b,10111111b,10111110b
	defb 00000000b,00000000b,00000000b,00000000b,10111111b,10111111b,10111111b,10111110b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,11111011b,11111011b,11111011b,11111010b
	defb 00000000b,00000000b,00000000b,00000000b,11111011b,11111011b,11111011b,11111010b
	defb 00000000b,00000000b,00000000b,00000000b,11111011b,11111011b,11111011b,11111010b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,10111111b,10111111b,10111111b,10111110b
	defb 00000000b,00000000b,00000000b,00000000b,10111111b,10111111b,10111111b,10111110b
	defb 00000000b,00000000b,00000000b,00000000b,10111111b,10111111b,10111111b,10111110b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,11111011b,11111011b,11111011b,11111010b
	defb 00000000b,00000000b,00000000b,00000000b,11111011b,11111011b,11111011b,11111010b
	defb 00000000b,00000000b,00000000b,00000000b,11111011b,11111011b,11111011b,11111010b

	defb 00000101b,00000101b,00000101b,01000101b,01000101b,01000101b,01000101b,01000101b
	defb 00000101b,00000101b,00000101b,01000101b,01000101b,01000101b,01000101b,01000101b

block_graphic_d
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,00011111b,11001000b,00011111b,11001000b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,10010000b,00001111b,10010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,10010000b,00001111b,10010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,10010000b,00001111b,10010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,10010000b,00001111b,10010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,10010000b,00001111b,10010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,10010000b,00001111b,10010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,10010000b,00001111b,10010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,10010000b,00001111b,10010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,00010000b,00001111b,00010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001111b,00010000b,00001111b,00010000b
	defb 00000000b,00000000b,00000000b,00000000b,00001110b,00010000b,00001110b,00010000b
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,00011111b,11001000b,00011111b,11001000b

	defb 00000101b,00000101b,00000101b,01000101b,01000101b,01000101b,01000101b,01000101b
	defb 00000101b,00000101b,00000101b,01000101b,01000101b,01000101b,01000101b,01000101b

block_graphic_e
	defb 00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b,00000000b
	defb 00000000b,00000000b,00000000b,00000000b,11011101b,01101110b,11011101b,01101110b
	defb 00000000b,00000000b,00000000b,00000001b,11111111b,11111111b,11111111b,11111111b
	defb 00000000b,00000000b,00000000b,00000001b,01110111b,11011111b,01110111b,11011111b
	defb 00000000b,00000000b,00000000b,00000000b,11111101b,11111011b,11111101b,11111011b
	defb 00000000b,00000000b,00000000b,00000001b,11101111b,11111111b,11101111b,11111111b
	defb 00000000b,00000000b,00000000b,00000000b,11111101b,11101111b,11111101b,11101110b
	defb 00100000b,10000000b,00100000b,10000000b,10110111b,01010110b,10110111b,01010110b
	defb 11001001b,00110100b,11001001b,00110100b,01001000b,10101001b,01001000b,10101000b
	defb 00000110b,00000011b,00000110b,00000011b,00101110b,11101111b,00101110b,11101110b
	defb 00000000b,00000000b,00000000b,00000000b,11101110b,11011011b,11101110b,11011010b
	defb 00000000b,00000000b,00000000b,00000000b,11011111b,01111011b,11011111b,01111010b
	defb 00000000b,00000000b,00000000b,00000001b,11011011b,01101101b,11011011b,01101101b
	defb 00000000b,00000000b,00000000b,00000001b,11101011b,11101101b,11101011b,11101101b
	defb 00000000b,00000000b,00000000b,00000001b,01101101b,11011111b,01101101b,11011111b
	defb 00000000b,00000000b,00000000b,00000001b,01111101b,11011011b,01111101b,11011011b

	defb 00000100b,00000100b,00000100b,00000100b,00000100b,00000100b,00000100b,00000100b
	defb 00001110b,00001110b,00001110b,00001110b,00001110b,00001110b,00001110b,00001110b

pattern_graphics
pattern_graphic_a
	defb 01000001b
	defb 00000001b
	defb 10000000b
	defb 01000000b
	defb 00100001b
	defb 00010010b
	defb 00001000b
	defb 00000100b
	defb 00010100b
	defb 00010000b
	defb 00001000b
	defb 00000100b
	defb 00010010b
	defb 00100001b
	defb 10000000b
	defb 01000000b

screen_task_list
screen_task_list_a
REPT 16, block_row		;pixel row of blocks
REPT 11, screen_row		;16-pixel multiple on screen
	ROW_NUM DEFL ((screen_row * 16) + block_row)
	defw (49152 + 32 + (32*((ROW_NUM) & 192)) + (4*((ROW_NUM) & 56)) + (256*((ROW_NUM) & 7)))	;end address of pixel row for first block
ENDM
ENDM
REPT 2, block_row		;char row of blocks
REPT 11, screen_row		;2-char multiple on screen
	ROW_NUM DEFL ((screen_row * 2) + block_row)
	defw (55296 + 32 + (32*ROW_NUM))					;end address of attr row for half of block
ENDM
ENDM
screen_task_list_b
REPT 16, block_row		;pixel row of blocks
	ROW_NUM DEFL (176 + block_row)
	defw (49152 + 32 + (32*((ROW_NUM) & 192)) + (4*((ROW_NUM) & 56)) + (256*((ROW_NUM) & 7)))	;end address of pixel row for first block
ENDM
REPT 2, block_row		;char row of blocks
	ROW_NUM DEFL (22 + block_row)
	defw (55296 + 32 + (32*ROW_NUM))					;end address of attr row for top half of block
ENDM
end_screen_task_list

map_data
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,1,0,0,0,0,0,1,0,1,0,1,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,1,1,1,0,1,1,1,0,1,1,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,1,1,1,1,0,0,0,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,1,1,0,0,0,1,1,0,0,0,0,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,1,1,1,1,1,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,1,0,0,0,0,0,0,0,1,0,1,0,0,0,1,0,1,0,0,0,0,0,0,1,0,0,0,1,0,0  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,1,1,0,1,1,0,0,0,1,1,0,0,0,0,1,0,1,0,1,0,1,0,0,0,0,0,0,0,1,1,0,1,1,0,0,0,1,1,0,1,1,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 1,1,1,1,1,0,0,0,0,1,1,1,1,1,0,1,1,1,1,1,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,1,1,0,1,1,0,0,0,1,1,0,0,0,0,1,1,1,0,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,1,1,1,1,1,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,1,0,1,0,0,0,0,0,0,1,0,1,0,0,0,1,0,1,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,1,0,1,0,0,1,0,0,0,0,0,1  ,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0  ,0
	defb 1, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1, 0,0,0,0,1,1,0,0,0,0,1,0,1,0,1,1, 1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,0,0,0,1,0,0,1,1,1,1,0,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,1,1  ,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1  ,1

push_table
	PUSH AF
	PUSH BC
	PUSH HL
	PUSH DE

push_set_list
	defw push_set_00
	defw push_set_01
	defw push_set_02
	defw push_set_03
	defw push_set_04
	defw push_set_05
	defw push_set_06
	defw push_set_07
	defw push_set_08
	defw push_set_09
	defw push_set_10
	defw push_set_11

END entry_point			;Tells PASMO to stop and build the TAP using that entry point label above
